@@ -41,6 +41,21 @@ enum SubCommand {
     MockDeqp(MockDeqp),
 }
 
+#[derive(Debug, StructOpt)]
+pub struct DeqpRunnerGlobalOptions {
+    #[structopt(
+        long,
+        help = "Optional path to store the deqp-vk .shader_cache files.  Must not be shared with any other deqp-runner invocations in progress."
+    )]
+    shader_cache_dir: Option<PathBuf>,
+
+    #[structopt(
+        long,
+        help = "Optional path to executor/testlog-to-xml, for converting QPA files to usable XML"
+    )]
+    testlog_to_xml: Option<PathBuf>,
+}
+
 #[derive(Debug, StructOpt)]
 pub struct Run {
     #[structopt(long, help = "path to deqp binary")]
@@ -53,13 +68,10 @@ pub struct Run {
     common: CommandLineRunOptions,
 
     #[structopt(flatten)]
-    deqp_config: DeqpRunConfig,
+    deqp_global: DeqpRunnerGlobalOptions,
 
-    #[structopt(
-        long,
-        help = "Optional path to executor/testlog-to-xml, for converting QPA files to usable XML"
-    )]
-    testlog_to_xml: Option<PathBuf>,
+    #[structopt(flatten)]
+    deqp_config: DeqpRunConfig,
 }
 
 #[derive(Debug, StructOpt)]
@@ -82,11 +94,8 @@ pub struct Suite {
     #[structopt(flatten)]
     common: CommandLineRunOptions,
 
-    #[structopt(
-        long,
-        help = "Optional path to executor/testlog-to-xml, for converting QPA files to usable XML"
-    )]
-    testlog_to_xml: Option<PathBuf>,
+    #[structopt(flatten)]
+    deqp_global: DeqpRunnerGlobalOptions,
 }
 
 // CLI/toml options for a dEQP run
@@ -176,13 +185,18 @@ impl DeqpTomlConfig {
     fn deqp(
         &self,
         run: &CommandLineRunOptions,
-        qpa_to_xml: Option<PathBuf>,
+        deqp_global: &DeqpRunnerGlobalOptions,
     ) -> Result<DeqpCommand> {
         Ok(DeqpCommand {
             deqp: self.deqp.clone(),
             args: self.deqp_config.deqp_args.clone(),
             config: TestConfiguration::from_suite_config(run, &self.sub_config)?,
-            qpa_to_xml,
+            shader_cache_dir: deqp_global
+                .shader_cache_dir
+                .as_ref()
+                .unwrap_or(&run.output_dir)
+                .clone(),
+            qpa_to_xml: deqp_global.testlog_to_xml.clone(),
             prefix: self.prefix.to_owned(),
         })
     }
@@ -363,7 +377,7 @@ fn main() -> Result<()> {
                 deqp_config: run.deqp_config,
                 prefix: "".to_owned(),
             };
-            let deqp = config.deqp(&run.common, run.testlog_to_xml)?;
+            let deqp = config.deqp(&run.common, &run.deqp_global)?;
 
             let results = parallel_test(std::io::stdout(), config.test_groups(&deqp, &[])?)?;
             process_results(&results, &run.common.output_dir, run.common.summary_limit)?;
@@ -401,7 +415,7 @@ fn main() -> Result<()> {
 
             let mut deqp = Vec::new();
             for config in &deqp_configs {
-                deqp.push(config.deqp(&suite.common, suite.testlog_to_xml.clone())?);
+                deqp.push(config.deqp(&suite.common, &suite.deqp_global)?);
             }
 
             let mut piglit = Vec::new();
