@@ -1,6 +1,6 @@
 use crate::parse_deqp::{parse_deqp_results_with_timeout, DeqpStatus};
 use crate::runner_results::*;
-use crate::{TestCase, TestCommand, TestConfiguration};
+use crate::{runner_thread_index, TestCase, TestCommand, TestConfiguration};
 use anyhow::{Context, Result};
 use log::*;
 use regex::Regex;
@@ -13,6 +13,7 @@ use std::process::{Command, Stdio};
 
 pub struct DeqpCommand {
     pub deqp: PathBuf,
+    pub shader_cache_dir: PathBuf,
     pub args: Vec<String>,
     pub config: TestConfiguration,
     pub qpa_to_xml: Option<PathBuf>,
@@ -261,7 +262,7 @@ impl TestCommand for DeqpCommand {
     fn run(
         &self,
         caselist_state: &CaselistState,
-        tests_o: &[&TestCase],
+        tests: &[&TestCase],
     ) -> Result<Vec<RunnerResult>> {
         let caselist_path = self
             .caselist_file_path(caselist_state, "caselist.txt")
@@ -270,18 +271,12 @@ impl TestCommand for DeqpCommand {
             .caselist_file_path(caselist_state, "qpa")
             .context("qpa path")?;
         let cache_path = self
-            .config
-            .output_dir
+            .shader_cache_dir
             .canonicalize()
             .context("cache path")?
-            .join(format!("t{}.shader_cache", thread_id::get()));
+            .join(format!("t{}.shader_cache", runner_thread_index()?));
 
-        let mut tests: Vec<&TestCase> = Vec::new();
-        for test in tests_o {
-            tests.push(test);
-        }
-
-        write_caselist_file(&caselist_path, tests.as_slice()).context("writing caselist file")?;
+        write_caselist_file(&caselist_path, tests).context("writing caselist file")?;
 
         let mut args: Vec<String> = Vec::new();
 
@@ -317,6 +312,7 @@ impl TestCommand for DeqpCommand {
             .stderr(Stdio::piped())
             .stdin(Stdio::null())
             .args(args)
+            .env("DEQP_RUNNER_THREAD", runner_thread_index()?.to_string())
             // Disable MESA_DEBUG output by default for debug Mesa builds, which
             // otherwise fills the logs with warnings about GL errors that are
             // thrown (you're running deqp!  Of course it makes GL errors!)
@@ -342,8 +338,10 @@ impl TestCommand for DeqpCommand {
         let caselist_results = caselist_results.context("parsing results")?;
         let mut deqp_results = caselist_results.results;
 
-        for result in &mut deqp_results {
-            result.name = format!("{}{}", self.prefix, result.name);
+        if !self.prefix.is_empty() {
+            for result in &mut deqp_results {
+                result.name = self.prefix().to_owned() + &result.name;
+            }
         }
 
         let stderr: Vec<String> = BufReader::new(child.stderr.as_mut().context("opening stderr")?)
@@ -367,7 +365,7 @@ impl TestCommand for DeqpCommand {
             error!("dEQP error: {}", line);
         }
 
-        let mut save_log = false;
+        let mut save_log = deqp_results.is_empty();
         let mut results: Vec<RunnerResult> = Vec::new();
         for result in deqp_results {
             let status = self.translate_result(&result, caselist_state);
@@ -387,7 +385,7 @@ impl TestCommand for DeqpCommand {
             results.push(RunnerResult {
                 test: result.name,
                 status,
-                duration: result.duration,
+                duration: result.duration.as_secs_f32(),
                 subtest: false,
             });
         }
