@@ -297,7 +297,7 @@ pub trait TestCommand: Send + Sync {
     }
 
     fn baseline_status(&self, test: &str) -> Option<RunnerStatus> {
-        self.baseline().tests.get(test).map(|x| x.status)
+        self.baseline().get(test).map(|x| x.status)
     }
 
     fn translate_result(
@@ -364,14 +364,24 @@ pub trait TestCommand: Send + Sync {
         Ok(results)
     }
 
-    fn process_caselist(&self, tests: &[TestCase], caselist_id: u32) -> Result<Vec<RunnerResult>> {
+    fn process_caselist(
+        &self,
+        tests: Vec<TestCase>,
+        caselist_id: u32,
+    ) -> Result<Vec<RunnerResult>> {
         let mut caselist_results: Vec<RunnerResult> = Vec::new();
         let mut remaining_tests = Vec::new();
         for test in tests {
-            let test = test.clone();
-            if self.skip_test(format!("{}{}", self.prefix(), &test.name()).as_str()) {
+            // Get the test name with the group prefix.
+            let name = if !self.prefix().is_empty() {
+                self.prefix().to_owned() + test.name()
+            } else {
+                test.name().to_owned()
+            };
+
+            if self.skip_test(&name) {
                 caselist_results.push(RunnerResult {
-                    test: format!("{}{}", self.prefix(), test.name()),
+                    test: name,
                     status: RunnerStatus::Skip,
                     duration: Default::default(),
                     subtest: false,
@@ -419,7 +429,7 @@ pub trait TestCommand: Send + Sync {
 
                     for test in remaining_tests {
                         caselist_results.push(RunnerResult {
-                            test: format!("{}{}", self.prefix(), &test.name()),
+                            test: self.prefix().to_owned() + test.name(),
                             status: RunnerStatus::Missing,
                             duration: Default::default(),
                             subtest: false,
@@ -476,6 +486,11 @@ pub trait TestCommand: Send + Sync {
             let group_len = usize::min(usize::max(remaining / 32, min), tests_per_group);
             remaining -= group_len;
 
+            if remaining == 0 {
+                // Free the memory remaining in the original test vector, because split_off(0) won't.
+                tests.shrink_to_fit();
+            }
+
             test_groups.push((self, tests.split_off(remaining)));
         }
 
@@ -541,17 +556,7 @@ fn results_collection<W: Write>(
         match group_results {
             Ok(group_results) => {
                 for result in group_results {
-                    if run_results.tests.contains_key(&result.test) {
-                        error!(
-                            "Duplicate test result for {}, marking test failed",
-                            &result.test
-                        );
-                        let mut fail_result = result;
-                        fail_result.status = RunnerStatus::Fail;
-                        run_results.record_result(fail_result);
-                    } else {
-                        run_results.record_result(result);
-                    }
+                    run_results.record_result(result);
                 }
             }
             Err(e) => {
@@ -596,7 +601,7 @@ pub fn parallel_test(
             .enumerate()
             .par_bridge()
             .try_for_each_with(sender, |sender, (i, (deqp, tests))| {
-                sender.send(deqp.process_caselist(&tests, i as u32))
+                sender.send(deqp.process_caselist(tests, i as u32))
             })
             .unwrap();
 
@@ -609,6 +614,10 @@ pub fn parallel_test(
     Ok(run_results)
 }
 
+pub fn runner_thread_index() -> Result<usize> {
+    rayon::current_thread_index().context("getting thread id within rayon global thread pool")
+}
+
 // Parses a deqp-runner regex set list.  We ignore empty lines and lines starting with "#", so you can
 // leave notes in your skips/flakes lists about why.
 pub fn parse_regex_set<I, S>(exprs: I) -> Result<RegexSet>
