@@ -254,7 +254,7 @@ fn parse_piglit_command(
     // separated by ','.  This still feels fragile.
     let mut all = command[1..(len - 1)]
         .split(',')
-        .map(|arg| arg.replace("'", ""));
+        .map(|arg| arg.replace('\'', ""));
     let binary = all.next().context("Getting binary")?;
     let mut args: Vec<String> = all
         .map(|arg| arg.trim().to_string())
@@ -284,7 +284,7 @@ fn parse_piglit_command(
 
 /* Replace ',' to avoid conflicts in the csv file */
 pub fn piglit_sanitize_test_name(test: &str) -> String {
-    test.replace(",", "-")
+    test.replace(',', "-")
 }
 
 pub fn parse_piglit_xml_testlist(
@@ -359,8 +359,8 @@ pub fn parse_piglit_xml_testlist(
                     name: test_name,
                     binary: "asmparsertest".to_string(),
                     args: vec![
-                        get_option_value(&test, "type_")?.replace("'", ""),
-                        get_option_value(&test, "filename")?.replace("'", ""),
+                        get_option_value(&test, "type_")?.replace('\'', ""),
+                        get_option_value(&test, "filename")?.replace('\'', ""),
                     ],
                 }));
             }
