@@ -2,7 +2,8 @@ use crate::parse_deqp::DeqpStatus;
 use anyhow::{Context, Result};
 use log::*;
 use regex::Regex;
-use std::collections::HashMap;
+use std::borrow::Borrow;
+use std::collections::HashSet;
 use std::fmt;
 use std::io::prelude::*;
 use std::io::{BufReader, BufWriter};
@@ -173,7 +174,7 @@ impl fmt::Display for RunnerStatus {
 pub struct RunnerResult {
     pub test: String,
     pub status: RunnerStatus,
-    pub duration: Duration,
+    pub duration: f32,
     pub subtest: bool,
 }
 
@@ -184,6 +185,29 @@ impl PartialEq for RunnerResult {
     }
 }
 
+pub struct RunnerResultNameHash(RunnerResult);
+
+// Use the test name as the hash key, which lets us store a HashSet<RunnerResult> intead of HashMap<String,RunnerResult>.
+impl core::hash::Hash for RunnerResultNameHash {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.0.test.hash(state);
+    }
+}
+
+impl PartialEq for RunnerResultNameHash {
+    fn eq(&self, other: &Self) -> bool {
+        self.0.test == other.0.test
+    }
+}
+
+impl Eq for RunnerResultNameHash {}
+
+impl Borrow<str> for RunnerResultNameHash {
+    fn borrow(&self) -> &str {
+        &self.0.test
+    }
+}
+
 #[derive(Eq, Clone, Copy, Hash, PartialEq)]
 pub struct CaselistState {
     pub caselist_id: u32,
@@ -266,7 +290,7 @@ impl fmt::Display for ResultCounts {
 }
 
 pub struct RunnerResults {
-    pub tests: HashMap<String, RunnerResult>,
+    pub tests: HashSet<RunnerResultNameHash>,
     pub result_counts: ResultCounts,
     pub time: Instant,
 }
@@ -276,46 +300,54 @@ impl RunnerResults {
         Default::default()
     }
 
+    pub fn get(&self, test: &str) -> Option<&RunnerResult> {
+        self.tests.get(test).map(|x| &x.0)
+    }
+
     pub fn record_result(&mut self, result: RunnerResult) {
-        self.result_counts.increment(result.status);
-        if !result.subtest {
+        let mut result = result;
+
+        if self.get(&result.test).is_some() {
+            error!(
+                "Duplicate test result for {}, marking test failed",
+                &result.test
+            );
+            result.status = RunnerStatus::Fail;
+        } else if !result.subtest {
             self.result_counts.total += 1;
         }
-
-        self.tests.insert(result.test.clone(), result);
+        self.result_counts.increment(result.status);
+        self.tests.replace(RunnerResultNameHash(result));
     }
 
     pub fn is_success(&self) -> bool {
-        self.tests
-            .iter()
-            .all(|(_, result)| result.status.is_success())
+        self.tests.iter().all(|result| result.0.status.is_success())
+    }
+
+    /// Returns a list of references to the results, sorted by test name.
+    pub fn sorted_results(&self) -> Vec<&RunnerResult> {
+        let mut sorted: Vec<_> = self.tests.iter().map(|x| &x.0).collect();
+        sorted.sort_by_key(|x| &x.test);
+        sorted
     }
 
     pub fn write_results<W: Write>(&self, writer: &mut W) -> Result<()> {
-        let mut sorted: Vec<_> = self.tests.iter().collect();
-        sorted.sort_by(|x, y| (*x).0.cmp((*y).0));
-
         let mut writer = BufWriter::new(writer);
-        for (test, result) in sorted {
+        for result in self.sorted_results() {
             writeln!(
                 writer,
                 "{},{},{}",
-                test,
-                result.status,
-                result.duration.as_secs_f32()
+                result.test, result.status, result.duration
             )?;
         }
         Ok(())
     }
 
     pub fn write_failures(&self, writer: &mut impl Write) -> Result<()> {
-        let mut sorted: Vec<_> = self.tests.iter().collect();
-        sorted.sort_by(|x, y| (*x).0.cmp((*y).0));
-
         let mut writer = BufWriter::new(writer);
-        for (test, result) in sorted {
+        for result in self.sorted_results() {
             if !result.status.is_success() {
-                writeln!(writer, "{},{}", test, result.status,)?;
+                writeln!(writer, "{},{}", result.test, result.status)?;
             }
         }
         Ok(())
@@ -327,9 +359,6 @@ impl RunnerResults {
         options: &JunitGeneratorOptions,
     ) -> Result<()> {
         use junit_report::*;
-        let mut sorted: Vec<_> = self.tests.iter().collect();
-        sorted.sort_by(|x, y| (*x).0.cmp((*y).0));
-
         let limit = if options.limit == 0 {
             std::usize::MAX
         } else {
@@ -337,15 +366,20 @@ impl RunnerResults {
         };
 
         let mut testcases = Vec::new();
-        for (test, result) in sorted.iter().take(limit) {
+        for result in self.sorted_results().iter().take(limit) {
             let tc = if !result.status.is_success() {
-                let message = options.template.replace("{{testcase}}", test);
+                let message = options.template.replace("{{testcase}}", &result.test);
 
                 let type_ = format!("{}", result.status);
 
-                junit_report::TestCase::failure(test, Duration::seconds(0), &type_, &message)
+                junit_report::TestCase::failure(
+                    &result.test,
+                    Duration::seconds(0),
+                    &type_,
+                    &message,
+                )
             } else {
-                junit_report::TestCase::success(test, Duration::seconds(0))
+                junit_report::TestCase::success(&result.test, Duration::seconds(0))
             };
             testcases.push(tc);
         }
@@ -375,14 +409,14 @@ impl RunnerResults {
                 let result = RunnerResult {
                     test: cap[1].to_string(),
                     status: cap[2].parse()?,
-                    duration: Duration::default(),
+                    duration: 0.0,
                     subtest: false,
                 };
 
                 // If you have more than one result for a test in the CSV,
                 // something has gone wrong (probably human error writing a
                 // baseline list)
-                if results.tests.contains_key(&result.test) {
+                if results.get(&result.test).is_some() {
                     anyhow::bail!("Found duplicate result for {} at line {}", line, lineno);
                 }
 
@@ -438,23 +472,23 @@ impl RunnerResults {
         let mut slowest: Vec<_> = self
             .tests
             .iter()
-            .map(|(test, result)| (test, result.duration))
+            .map(|result| (&result.0.test, result.0.duration))
             .collect();
         // Sorting on duration and reversing because you can't negate a duration and you can't easily
         // sort on a f32.
-        slowest.sort_by_key(|x| x.1);
+        slowest.sort_by(|a, b| a.1.partial_cmp(&b.1).unwrap());
         slowest.reverse();
 
         println!();
         println!("Slowest tests:");
         for test in slowest.iter().take(5) {
-            println!("  {} ({:.02}s)", test.0, test.1.as_secs_f32());
+            println!("  {} ({:.02}s)", test.0, test.1);
         }
 
         let mut flakes: Vec<_> = self
             .tests
             .iter()
-            .map(|(name, result)| (name, result.status))
+            .map(|result| (&result.0.test, result.0.status))
             .filter(|(_, status)| *status == RunnerStatus::Flake)
             .collect();
         if !flakes.is_empty() {
@@ -473,7 +507,7 @@ impl RunnerResults {
         let mut fails: Vec<_> = self
             .tests
             .iter()
-            .map(|(name, result)| (name, result.status))
+            .map(|result| (&result.0.test, result.0.status))
             .filter(|(_, status)| !status.is_success())
             .collect();
         if !fails.is_empty() {
@@ -593,21 +627,21 @@ piglit@crashy@test,Crash"
                 .as_bytes(),
         )?;
         assert_eq!(
-            results.tests.get("dEQP-GLES2.info.version"),
+            results.get("dEQP-GLES2.info.version"),
             Some(RunnerResult {
                 test: "dEQP-GLES2.info.version".to_string(),
                 status: RunnerStatus::Fail,
-                duration: std::time::Duration::new(0, 0),
+                duration: 0.0,
                 subtest: false,
             })
             .as_ref()
         );
         assert_eq!(
-            results.tests.get("piglit@crashy@test"),
+            results.get("piglit@crashy@test"),
             Some(RunnerResult {
                 test: "piglit@crashy@test".to_string(),
                 status: RunnerStatus::Crash,
-                duration: std::time::Duration::new(0, 0),
+                duration: 0.0,
                 subtest: false,
             })
             .as_ref()
@@ -653,7 +687,7 @@ dEQP-GLES2.info.version,Pass"
         results.record_result(RunnerResult {
             test: test.to_string(),
             status,
-            duration: Duration::new(0, 0),
+            duration: 0.0,
             subtest: false,
         });
     }
