@@ -210,7 +210,7 @@ fn mocked_deqp_runner<S: AsRef<str>>(tests: Vec<S>) -> RunnerResults {
 }
 
 fn result_status<S: AsRef<str>>(results: &RunnerResults, test: S) -> RunnerStatus {
-    results.tests.get(test.as_ref()).unwrap().status
+    results.get(test.as_ref()).unwrap().status
 }
 
 #[derive(Default)]
@@ -760,19 +760,19 @@ fn suite_prefix_test() {
     assert_eq!(results.result_counts.unexpected_pass, 1);
 
     assert_eq!(
-        results.tests["dEQP-GLES2.test.p.1"].status,
+        results.get("dEQP-GLES2.test.p.1").unwrap().status,
         RunnerStatus::Pass
     );
     assert_eq!(
-        results.tests["variant1-dEQP-GLES2.test.p.1"].status,
+        results.get("variant1-dEQP-GLES2.test.p.1").unwrap().status,
         RunnerStatus::Pass
     );
     assert_eq!(
-        results.tests["variant1-dEQP-GLES2.test.p.2"].status,
+        results.get("variant1-dEQP-GLES2.test.p.2").unwrap().status,
         RunnerStatus::Skip
     );
     assert_eq!(
-        results.tests["variant2-dEQP-GLES2.test.p.1"].status,
+        results.get("variant2-dEQP-GLES2.test.p.1").unwrap().status,
         RunnerStatus::UnexpectedPass
     );
 }
@@ -800,11 +800,11 @@ fn suite_prefix_skips() {
     assert_eq!(results.result_counts.fail, 0);
 
     assert_eq!(
-        results.tests["dEQP-GLES2.test.p.1"].status,
+        results.get("dEQP-GLES2.test.p.1").unwrap().status,
         RunnerStatus::Skip
     );
     assert_eq!(
-        results.tests["variant-dEQP-GLES2.test.p.1"].status,
+        results.get("variant-dEQP-GLES2.test.p.1").unwrap().status,
         RunnerStatus::Skip
     );
 }
@@ -824,7 +824,7 @@ fn suite_missing() {
     assert_eq!(results.result_counts.fail, 0);
 
     assert_eq!(
-        results.tests["variant-dEQP-GLES2.test.m.1"].status,
+        results.get("variant-dEQP-GLES2.test.m.1").unwrap().status,
         RunnerStatus::Missing
     );
 }
@@ -849,7 +849,7 @@ fn suite_includes() {
     let results = results.results.unwrap();
     assert_eq!(results.result_counts.pass, 1);
     assert_eq!(
-        results.tests["dEQP-GLES2.test.p.1.a.c"].status,
+        results.get("dEQP-GLES2.test.p.1.a.c").unwrap().status,
         RunnerStatus::Pass
     );
 }
