@@ -1,6 +1,6 @@
 use ::deqp_runner::RunnerResults;
 use anyhow::{Context, Result};
-use std::io::prelude::*;
+use std::{io::prelude::*, path::PathBuf};
 
 /// Integration test binary.  See https://matklad.github.io/2021/02/27/delete-cargo-integration-tests.html
 mod deqp_runner;
@@ -32,3 +32,9 @@ pub fn lines_tempfile<S: AsRef<str>, I: IntoIterator<Item = S>>(
     }
     Ok(file.into_temp_path())
 }
+
+// Returns the path to a test resource checked into git.
+pub fn test_resource_path(filename: &str) -> PathBuf {
+    let manifest = PathBuf::from(env!("CARGO_MANIFEST_DIR"));
+    manifest.join("resources/test").join(filename)
+}
